package javax.swing;

import java.awt.event.KeyEvent;
import java.awt.Rectangle;
import java.awt.Component;
import java.awt.Point;
import sun.awt.AWTAccessor;
import java.awt.event.MouseEvent;
import javax.swing.event.ChangeListener;
import sun.swing.SwingUtilities2;
import sun.awt.AppContext;
import javax.swing.event.EventListenerList;
import javax.swing.event.ChangeEvent;
import java.util.Vector;

public class MenuSelectionManager
{
    private Vector<MenuElement> selection;
    private static final boolean TRACE = false;
    private static final boolean VERBOSE = false;
    private static final boolean DEBUG = false;
    private static final StringBuilder MENU_SELECTION_MANAGER_KEY;
    protected transient ChangeEvent changeEvent;
    protected EventListenerList listenerList;
    
    public MenuSelectionManager() {
        this.selection = new Vector();
        this.changeEvent = null;
        this.listenerList = new EventListenerList();
    }
    
    public static MenuSelectionManager defaultManager() {
        synchronized (MenuSelectionManager.MENU_SELECTION_MANAGER_KEY) {
            AppContext appContext = AppContext.getAppContext();
            MenuSelectionManager value = (MenuSelectionManager)appContext.get(MenuSelectionManager.MENU_SELECTION_MANAGER_KEY);
            if (value == null) {
                value = new MenuSelectionManager();
                appContext.put(MenuSelectionManager.MENU_SELECTION_MANAGER_KEY, value);
                Object value2 = appContext.get(SwingUtilities2.MENU_SELECTION_MANAGER_LISTENER_KEY);
                if (value2 != null && value2 instanceof ChangeListener) {
                    value.addChangeListener((ChangeListener)value2);
                }
            }
            return value;
        }
    }
    
    public void setSelectedPath(MenuElement[] array) {
        int size = this.selection.size();
        int i = 0;
        if (array == null) {
            array = new MenuElement[0];
        }
        for (int index = 0; index < array.length && index < size && this.selection.elementAt(index) == array[index]; ++index) {
            ++i;
        }
        for (int j = size - 1; j >= i; --j) {
            MenuElement menuElement = (MenuElement)this.selection.elementAt(j);
            this.selection.removeElementAt(j);
            menuElement.menuSelectionChanged(false);
        }
        for (; i < array.length; ++i) {
            if (array[i] != null) {
                this.selection.addElement(array[i]);
                array[i].menuSelectionChanged(true);
            }
        }
        this.fireStateChanged();
    }
    
    public MenuElement[] getSelectedPath() {
        MenuElement[] array = new MenuElement[this.selection.size()];
        for (int i = 0; i < this.selection.size(); ++i) {
            array[i] = (MenuElement)this.selection.elementAt(i);
        }
        return array;
    }
    
    public void clearSelectedPath() {
        if (this.selection.size() > 0) {
            this.setSelectedPath((MenuElement[])null);
        }
    }
    
    public void addChangeListener(ChangeListener l) {
        this.listenerList.add(ChangeListener.class, l);
    }
    
    public void removeChangeListener(ChangeListener l) {
        this.listenerList.remove(ChangeListener.class, l);
    }
    
    public ChangeListener[] getChangeListeners() {
        return (ChangeListener[])this.listenerList.getListeners(ChangeListener.class);
    }
    
    protected void fireStateChanged() {
        Object[] listenerList = this.listenerList.getListenerList();
        for (int i = listenerList.length - 2; i >= 0; i -= 2) {
            if (listenerList[i] == ChangeListener.class) {
                if (this.changeEvent == null) {
                    this.changeEvent = new ChangeEvent(this);
                }
                ((ChangeListener)listenerList[i + 1]).stateChanged(this.changeEvent);
            }
        }
    }
    
    public void processMouseEvent(MouseEvent mouseEvent) {
        Point point = mouseEvent.getPoint();
        Component component = mouseEvent.getComponent();
        if (component != null && !component.isShowing()) {
            return;
        }
        int id = mouseEvent.getID();
        int modifiers = mouseEvent.getModifiers();
        if ((id == 504 || id == 505) && (modifiers & 0x1C) != 0x0) {
            return;
        }
        if (component != null) {
            SwingUtilities.convertPointToScreen(point, component);
        }
        int x = point.x;
        int y = point.y;
        Vector<Object> vector = (Vector)this.selection.clone();
        int size = vector.size();
        for (int n = 0, index = size - 1; index >= 0 && n == 0; --index) {
            MenuElement[] subElements = ((MenuElement)vector.elementAt(index)).getSubElements();
            MenuElement[] array = null;
            for (int n2 = 0; n2 < subElements.length && n == 0; ++n2) {
                if (subElements[n2] != null) {
                    Component component2 = subElements[n2].getComponent();
                    if (component2.isShowing()) {
                        int n3;
                        int n4;
                        if ((component2 instanceof JComponent)) {
                            n3 = component2.getWidth();
                            n4 = component2.getHeight();
                        }
                        else {
                            Rectangle bounds = component2.getBounds();
                            n3 = bounds.width;
                            n4 = bounds.height;
                        }
                        point.x = x;
                        point.y = y;
                        SwingUtilities.convertPointFromScreen(point, component2);
                        if (point.x >= 0 && point.x < n3 && point.y >= 0 && point.y < n4) {
                            if (array == null) {
                                array = new MenuElement[index + 2];
                                for (int i = 0; i <= index; ++i) {
                                    array[i] = (MenuElement)vector.elementAt(i);
                                }
                            }
                            array[index + 1] = subElements[n2];
                            MenuElement[] selectedPath = this.getSelectedPath();
                            if (selectedPath[selectedPath.length - 1] != array[index + 1] && (selectedPath.length < 2 || selectedPath[selectedPath.length - 2] != array[index + 1])) {
                                MouseEvent o = new MouseEvent(selectedPath[selectedPath.length - 1].getComponent(), 505, mouseEvent.getWhen(), mouseEvent.getModifiers(), point.x, point.y, mouseEvent.getXOnScreen(), mouseEvent.getYOnScreen(), mouseEvent.getClickCount(), mouseEvent.isPopupTrigger(), 0);
                                AWTAccessor.MouseEventAccessor mouseEventAccessor = AWTAccessor.getMouseEventAccessor();
                                mouseEventAccessor.setCausedByTouchEvent(o, mouseEventAccessor.isCausedByTouchEvent(mouseEvent));
                                selectedPath[selectedPath.length - 1].processMouseEvent(o, array, this);
                                MouseEvent o2 = new MouseEvent(component2, 504, mouseEvent.getWhen(), mouseEvent.getModifiers(), point.x, point.y, mouseEvent.getXOnScreen(), mouseEvent.getYOnScreen(), mouseEvent.getClickCount(), mouseEvent.isPopupTrigger(), 0);
                                mouseEventAccessor.setCausedByTouchEvent(o2, mouseEventAccessor.isCausedByTouchEvent(mouseEvent));
                                subElements[n2].processMouseEvent(o2, array, this);
                            }
                            MouseEvent o3 = new MouseEvent(component2, mouseEvent.getID(), mouseEvent.getWhen(), mouseEvent.getModifiers(), point.x, point.y, mouseEvent.getXOnScreen(), mouseEvent.getYOnScreen(), mouseEvent.getClickCount(), mouseEvent.isPopupTrigger(), 0);
                            AWTAccessor.MouseEventAccessor mouseEventAccessor2 = AWTAccessor.getMouseEventAccessor();
                            mouseEventAccessor2.setCausedByTouchEvent(o3, mouseEventAccessor2.isCausedByTouchEvent(mouseEvent));
                            subElements[n2].processMouseEvent(o3, array, this);
                            n = 1;
                            mouseEvent.consume();
                        }
                    }
                }
            }
        }
        return;
    }
    
    private void printMenuElementArray(MenuElement[] array) {
        this.printMenuElementArray(array, false);
    }
    
    private void printMenuElementArray(MenuElement[] array, boolean b) {
        System.out.println("Path is(");
        for (int i = 0; i < array.length; ++i) {
            for (int j = 0; j <= i; ++j) {
                System.out.print("  ");
            }
            MenuElement obj = array[i];
            if ((obj instanceof JMenuItem)) {
                System.out.println(((JMenuItem)obj).getText() + ", ");
            }
            else if ((obj instanceof JMenuBar)) {
                System.out.println("JMenuBar, ");
            }
            else if ((obj instanceof JPopupMenu)) {
                System.out.println("JPopupMenu, ");
            }
            else if (obj == null) {
                System.out.println("NULL , ");
            }
            else {
                System.out.println("" + obj + ", ");
            }
        }
        System.out.println(")");
        if (b) {
            Thread.dumpStack();
        }
    }
    
    public Component componentForPoint(Component component, Point point) {
        SwingUtilities.convertPointToScreen(point, component);
        int x = point.x;
        int y = point.y;
        Vector<Object> vector = (Vector)this.selection.clone();
        for (int i = vector.size() - 1; i >= 0; --i) {
            MenuElement[] subElements = ((MenuElement)vector.elementAt(i)).getSubElements();
            for (int j = 0; j < subElements.length; ++j) {
                if (subElements[j] != null) {
                    Component component2 = subElements[j].getComponent();
                    if (component2.isShowing()) {
                        int n;
                        int n2;
                        if ((component2 instanceof JComponent)) {
                            n = component2.getWidth();
                            n2 = component2.getHeight();
                        }
                        else {
                            Rectangle bounds = component2.getBounds();
                            n = bounds.width;
                            n2 = bounds.height;
                        }
                        point.x = x;
                        point.y = y;
                        SwingUtilities.convertPointFromScreen(point, component2);
                        if (point.x >= 0 && point.x < n && point.y >= 0 && point.y < n2) {
                            return component2;
                        }
                    }
                }
            }
        }
        return null;
    }
    
    public void processKeyEvent(KeyEvent keyEvent) {
        MenuElement[] array = (MenuElement[])this.selection.toArray(new MenuElement[0]);
        int length = array.length;
        if (length < 1) {
            return;
        }
        for (int i = length - 1; i >= 0; --i) {
            MenuElement[] subElements = array[i].getSubElements();
            MenuElement[] array2 = null;
            for (int j = 0; j < subElements.length; ++j) {
                if (subElements[j] != null && subElements[j].getComponent().isShowing() && subElements[j].getComponent().isEnabled()) {
                    if (array2 == null) {
                        array2 = new MenuElement[i + 2];
                        System.arraycopy(array, 0, array2, 0, i + 1);
                    }
                    array2[i + 1] = subElements[j];
                    subElements[j].processKeyEvent(keyEvent, array2, this);
                    if (keyEvent.isConsumed()) {
                        return;
                    }
                }
            }
        }
        MenuElement[] array3 = { array[0] };
        array3[0].processKeyEvent(keyEvent, array3, this);
        if (keyEvent.isConsumed()) {
            return;
        }
        return;
    }
    
    public boolean isComponentPartOfCurrentMenu(Component component) {
        return this.selection.size() > 0 && this.isComponentPartOfCurrentMenu((MenuElement)this.selection.elementAt(0), component);
    }
    
    private boolean isComponentPartOfCurrentMenu(MenuElement menuElement, Component component) {
        if (menuElement == null) {
            return false;
        }
        if (menuElement.getComponent() == component) {
            return true;
        }
        MenuElement[] subElements = menuElement.getSubElements();
        int i = 0;
        while (i < subElements.length) {
            if (this.isComponentPartOfCurrentMenu(subElements[i], component)) {
                return true;
            }
            ++i;
        }
        return false;
    }
    
    static {
        MENU_SELECTION_MANAGER_KEY = new StringBuilder("javax.swing.MenuSelectionManager");
    }
}
